<?php

/**
 * Carerix Wordpress Plugin
 *
 *
 * @category Loader
 * @author Bogdan Ivanov
 * @copyright Copyright (c) 2013 Carerix.com (http://www.carerix.com)
 * @link http://www.carerix.com
 * @since Apr 11, 2013 10:07:38 AM
 * @version $Rev$
 */


/**
 * Loader class
 * Used to set up the autoloading mechanism of classes
 * 
 */
class Loader {
	
	private $baseDirectory = null;
	
	public function __construct($base_directory) 
	{
		$this->baseDirectory = $base_directory;
	}
	
	public function run()
	{
		$this->setIncludePath();
		$this->setBasicAutoload();
	}
	
	/**
	 * registers the autoload chain
	 */
	private function setBasicAutoload()
	{
		spl_autoload_register(array('Loader', 'autoloadConfig'), true);
		spl_autoload_register(array('Loader', 'autoloadCarerixApi'), true);
		spl_autoload_register(array('Loader', 'autoloadCarerixClasses'), true);
		spl_autoload_register(array('Loader', 'autoloadPear'), true);
	}
	
	/**
	 * registers the plugin include paths
	 */
	private function setIncludePath()
    {
        set_include_path(get_include_path() . PATH_SEPARATOR . dirname(__FILE__) );
		set_include_path(get_include_path() . PATH_SEPARATOR . dirname(__FILE__) . '/vendor'  );
		set_include_path(get_include_path() . PATH_SEPARATOR . dirname(__FILE__) . '/vendor/PEAR'  );
    }
	
	/**
	 * autoloader for a class from the Carerix plugin
	 * Note all plugin classes in wp-content/plugins/ are included.
	 * Todo: refactor if this is required
	 *
	 * @param string $classname
	 */
	public static  function autoloadConfig($classname) 
	{
		$classname = str_replace('_', DIRECTORY_SEPARATOR, $classname);
		$path = implode(DIRECTORY_SEPARATOR, 
				array(dirname(__FILE__), $classname.'.php'));
		if(file_exists($path)) {
			require_once($path);
		}
	}
	
	/**
	 * autoloader for classes from the Pear packages
	 * 
	 * @param string $classname
	 */
	public static function autoloadPear($classname)
	{
		$filename = str_replace('_', '/', $classname).'.php';
		
		$filename = implode(DIRECTORY_SEPARATOR, array(dirname(__FILE__), 'vendor','PEAR', $filename ));
		if(file_exists($filename))
		{
			require_once $filename;
		}
	}
	
	/**
	 * autoloader for classes from the Carerix Rest API
	 * 
	 * @param string $classname
	 */
	public static function autoloadCarerixApi($classname)
	{
		$apiRestLoader = implode(DIRECTORY_SEPARATOR, 
				array(dirname(__FILE__), 'vendor', 'Carerix' , 'Api', 'Rest', 'Loader.php'));
		require_once($apiRestLoader);

/** PHP 5.3 is outdated and deprecated */
//		$php53 = implode(DIRECTORY_SEPARATOR,
//				array(dirname(__FILE__), 'vendor', 'Carerix' ,'php5.3.php'));
//		require_once($php53);
		
		$classname = str_replace('_', DIRECTORY_SEPARATOR, $classname);
		$vendorPath = implode(DIRECTORY_SEPARATOR, 
				array(dirname(__FILE__), 'vendor', $classname.'.php'));
		if(file_exists($vendorPath))
		{
			require_once($vendorPath);
		}
	}

	/**
	 * autoloader for extra Carerix classes
	 *
	 * @param string $classname
	 */
	public static function autoloadCarerixClasses($classname)
	{
		$filename = basename(str_replace('\\', '/', $classname).'.php');

		$filename = implode(DIRECTORY_SEPARATOR, array(dirname(__FILE__), 'vendor','Carerix', 'Classes', $filename ));

		if(file_exists($filename))
		{
			require_once $filename;
		}
	}

}
